/* Dark Well - old-good game, mod of tetris.
 * Copyright (C) 2000 Boris Kozlov.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string.h>
#include <sys/stat.h>
#include <ncurses.h>
#include "score.h"
#include <errno.h>
#include <time.h>

#define SCORE_FILE "darkwell.score"

static unsigned char numofscore=0;

void makescorefile();

void openscore()
{
  FILE *f;
  unsigned char i;
  struct stat stat_buf;  
  
  if (!(f=fopen(SCORE_FILE,"r")))
    {
      printw("Couldn't open file %s, creating new. Press any key.\n",SCORE_FILE);
      getch();
      if (!(f=fopen(SCORE_FILE,"w+"))) {  //make empty file
        printw("Couldn't create file %s!\n",SCORE_FILE);
        getch();	
        exiting(1);
      }	
  }

  if (stat (SCORE_FILE, &stat_buf) == -1) { 
      printw("couldn't stat high-score file\n");
  }
  numofscore = stat_buf.st_size/sizeof (struct info);

  for (i=0;i<numofscore;i++) {
      fread (hi_scores[i].name, sizeof(x.name), 1, f);
      fread (&hi_scores[i].score, sizeof(x.score), 1, f);
      fread (&hi_scores[i].year, sizeof(x.score), 1, f);
      fread (&hi_scores[i].month, sizeof(x.score), 1, f);
      fread (&hi_scores[i].day, sizeof(x.score), 1, f);
  }
  fclose(f);
  
}; //openscore  

printscore()
{
  unsigned i,j;
  
  setcolor(15);
  for (i=0;i<10;i++) {
    move(i+3,1); 
    for (j=0;j<25;j++) printw(" ");
  }   
  
  if (numofscore!=0) 
    {
      for (i=0;i<numofscore;i++) 
	{
	  if (i==1) setcolor(10);	
	  if (i==5) setcolor(2);
	  mvaddstr(i+3,0,hi_scores[i].name);
	  mvprintw(i+3,10,"%i",hi_scores[i].score);    	  
	  mvprintw(i+3,16,"%i%c",hi_scores[i].day,'-');    
	  printw("%i%c",hi_scores[i].month,'-');    	  
	  printw("%i",hi_scores[i].year);    	  
	}
      refresh();    
    }
} //printscore

fillscorefile()
{
  FILE *f;
  int i;
  
  if (!(f=fopen(SCORE_FILE,"w"))) {
      printw("Couldn't open file %s!\n",SCORE_FILE);
      exiting(1);
  }

  for (i=0;i<numofscore;i++) {
      fwrite (hi_scores[i].name, sizeof(x.name), 1, f);
      fwrite (&hi_scores[i].score, sizeof(x.score), 1, f);
      fwrite (&hi_scores[i].year, sizeof(x.score), 1, f);
      fwrite (&hi_scores[i].month, sizeof(x.score), 1, f);
      fwrite (&hi_scores[i].day, sizeof(x.score), 1, f);
  }
  fclose(f);  
} //fillscorefile

void sort()
{
 unsigned char i=1,j=10;
 
 for (;i<=10;i++)
   for (j=10;j>=i;j--)
     if (hi_scores[j-1].score < hi_scores[j].score) {
       x=hi_scores[j-1];
       hi_scores[j-1]=hi_scores[j];
       hi_scores[j]=x;
     }
} //sort

void get_date(int *day, int *month, int *year)
{
  time_t curtime;
  struct tm *loctime;
  
  curtime = time (NULL);
  loctime = localtime (&curtime);

  *day=loctime->tm_mday;
  *month=loctime->tm_mon+1;
  *year=loctime->tm_year+1900;
}  

//----------------------------------------------------------------------------

int charokey(char c)
{
  switch(c) {
    case ' ' : 
    case '^' : 
    case '/' :         
    case '>' :             
    case '<' :                 
    case '=' : return(0);                    
    default : return(1);
  }
}

void readname(char *str)
{
 char *st2="Please input Your name : "; 
 int fin=0,c,pos=0,n=9;
 char erase=erasechar();
 unsigned y,x;

 setcolor(15);
 wr(st2,24,40-strlen(st2)/2);
 flushinp();
 curs_set(1); 
 setcolor(9);
 keypad(stdscr,1);
 noecho ();
 refresh ();
 do {

    while ((c=getch()) == ERR && errno == EINTR);
    switch (c) {
    case '\n':
    case '\r': fin=1; break;
    case ERR: //error
        break;
    default:
      if (c == erase && pos > 0) {
	  --pos; wr("--pos",0,70);
      } else if	(c == erase && pos == 0) {
        /*getyx(stdscr,y,x);*/ move(0,0);      
      } else if (pos < n && isprint(c) && charokey(c)) {
	str[pos++] = c;
	printw("%c",c);
      } 
    }
  } while (!fin);
  
  noecho ();
  keypad (stdscr,1);
  curs_set(0); 
  if (pos < n) str[pos] = '\0';
} //readname 

//--------------------------------------------------------------------------

void savescore(unsigned curscore)
{
  char *st;
  unsigned day,month,year;

  if (!(curscore <= hi_scores[9].score && numofscore==10)) {
    x.score=curscore; 
    get_date(&x.day,&x.month,&x.year);

    if (numofscore < 10) numofscore++;
   
    st=malloc(10);
    readname(st);
    if (strlen(st) > 0) strncpy(x.name,st,10); else strcpy(x.name,"Nobody");
    free(st);

    if (numofscore==1) hi_scores[0]=x;
    else {
      hi_scores[10]=x; 
      sort();
    }
    fillscorefile();
    printscore();
  }  
} //savescore

